@@ -48,9 +48,14 @@ struct praw_connection_t {
   pn_event_batch_t batch;
   struct addrinfo *addrinfo;         /* Resolved address list */
   struct addrinfo *ai;               /* Current connect address */
+  int current_arm;                   /* Active epoll io events */
+  bool armed;
   bool connected;
   bool disconnected;
-  bool batch_empty;
+  bool hup_detected;
+  bool read_check;
+  bool first_schedule;
+  char *taddr;
 };
 
 static void psocket_error(praw_connection_t *rc, int err, const char* msg) {
@@ -98,7 +103,8 @@ static void praw_connection_start(praw_connection_t *prc, int fd) {
     pclosefd(prc->task.proactor, fd);
   }
   ee->fd = fd;
-  ee->wanted = EPOLLIN | EPOLLOUT;
+  prc->current_arm = ee->wanted = EPOLLIN | EPOLLOUT;
+  prc->armed = true;
   start_polling(ee, efd);  // TODO: check for error
 }
 
@@ -144,6 +150,10 @@ static void praw_connection_init(praw_connection_t *prc, pn_proactor_t *p, pn_ra
 
   prc->connected = false;
   prc->disconnected = false;
+  prc->first_schedule = false;
+  prc->armed = false;
+  prc->current_arm = 0;
+  prc->taddr = NULL;
   prc->batch.next_event = pni_raw_batch_next;
 
   pmutex_init(&prc->rearm_mutex);
@@ -162,11 +172,23 @@ static void praw_connection_cleanup(praw_connection_t *prc) {
     task_finalize(&prc->task);
     if (prc->addrinfo)
       freeaddrinfo(prc->addrinfo);
+    free(prc->taddr);
     free(prc);
   }
   // else proactor_disconnect logic owns prc and its final free
 }
 
+static void praw_initiate_cleanup(praw_connection_t *prc) {
+  if (prc->armed) {
+    // Possible race with epoll event.  Wait for it to clear.
+    // Force EPOLLHUP callback if not already pending.
+    shutdown(prc->psocket.epoll_io.fd, SHUT_RDWR);
+    return;
+  }
+  pni_raw_finalize(&prc->raw_connection);
+  praw_connection_cleanup(prc);
+}
+
 pn_raw_connection_t *pn_raw_connection(void) {
   praw_connection_t *conn = (praw_connection_t*) calloc(1, sizeof(praw_connection_t));
   if (!conn) return NULL;
@@ -176,39 +198,48 @@ pn_raw_connection_t *pn_raw_connection(void) {
   return &conn->raw_connection;
 }
 
+// Call from pconnection_process with task lock held.
+// Return true if the socket is connecting and there are no Proton events to deliver.
+static bool praw_connection_first_connect_lh(praw_connection_t *prc) {
+  const char *host;
+  const char *port;
+
+  unlock(&prc->task.mutex);
+  size_t addrlen = strlen(prc->taddr);
+  char *addr_buf = (char*) alloca(addrlen+1);
+  pni_parse_addr(prc->taddr, addr_buf, addrlen+1, &host, &port);
+  // TODO: move this step to a separate worker thread that scales in response to multiple blocking DNS lookups.
+  int gai_error = pgetaddrinfo(host, port, 0, &prc->addrinfo);
+  lock(&prc->task.mutex);
+
+  if (!gai_error) {
+    prc->ai = prc->addrinfo;
+    praw_connection_maybe_connect_lh(prc); /* Start connection attempts */
+    if (prc->psocket.epoll_io.fd != -1 && !pni_task_wake_pending(&prc->task))
+      return true;
+  } else {
+    psocket_gai_error(prc, gai_error, "connect to ", prc->taddr);
+  }
+  return false;
+}
+
 void pn_proactor_raw_connect(pn_proactor_t *p, pn_raw_connection_t *rc, const char *addr) {
+  // Called from an arbitrary thread.  Do setup prior to getaddrinfo, then switch to a worker thread.
   assert(rc);
   praw_connection_t *prc = containerof(rc, praw_connection_t, raw_connection);
   praw_connection_init(prc, p, rc);
   // TODO: check case of proactor shutting down
 
   lock(&prc->task.mutex);
-  proactor_add(&prc->task);
-
-  bool notify = false;
-
-  const char *host;
-  const char *port;
   size_t addrlen = strlen(addr);
-  char *addr_buf = (char*) alloca(addrlen+1);
-  pni_parse_addr(addr, addr_buf, addrlen+1, &host, &port);
-
-  int gai_error = pgetaddrinfo(host, port, 0, &prc->addrinfo);
-  if (!gai_error) {
-    prc->ai = prc->addrinfo;
-    praw_connection_maybe_connect_lh(prc); /* Start connection attempts */
-    if (prc->disconnected) notify = schedule(&prc->task);
-  } else {
-    psocket_gai_error(prc, gai_error, "connect to ", addr);
-    prc->disconnected = true;
-    notify = schedule(&prc->task);
-    lock(&p->task.mutex);
-    notify |= schedule_if_inactive(p);
-    unlock(&p->task.mutex);
-  }
-
-  /* We need to issue INACTIVE on immediate failure */
+  prc->taddr = (char*) malloc(addrlen+1);
+  assert(prc->taddr); // TODO: memory safety
+  memcpy(prc->taddr, addr, addrlen+1);
+  prc->first_schedule = true; // Resume connection setup when next scheduled.
+  proactor_add(&prc->task);
+  bool notify = schedule(&prc->task);
   unlock(&prc->task.mutex);
+
   if (notify) notify_poller(p);
 }
 
@@ -243,10 +274,12 @@ void pn_listener_raw_accept(pn_listener_t *l, pn_raw_connection_t *rc) {
     praw_connection_connected_lh(prc);
   } else {
     psocket_error(prc, err, "pn_listener_accept");
+    pni_raw_connect_failed(&prc->raw_connection);
+    notify = schedule(&prc->task);
   }
 
   if (!l->task.working && listener_has_event(l)) {
-    notify = schedule(&l->task);
+    notify |= schedule(&l->task);
   }
   unlock(&prc->task.mutex);
   unlock(&l->task.mutex);
@@ -318,10 +351,7 @@ static pn_event_t *pni_raw_batch_next(pn_event_batch_t *batch) {
   unlock(&rc->task.mutex);
   if (waking) pni_raw_wake(raw);
 
-  pn_event_t *e = pni_raw_event_next(raw);
-  if (!e || pn_event_type(e) == PN_RAW_CONNECTION_DISCONNECTED)
-    rc->batch_empty = true;
-  return e;
+  return pni_raw_event_next(raw);
 }
 
 task_t *pni_psocket_raw_task(psocket_t* ps) {
@@ -350,8 +380,10 @@ task_t *pni_raw_connection_task(praw_connection_t *rc) {
   return &rc->task;
 }
 
-static long snd(int fd, const void* b, size_t s) {
-  return send(fd, b, s, MSG_NOSIGNAL | MSG_DONTWAIT);
+static long snd(int fd, const void* b, size_t s, bool more) {
+  int flags = MSG_NOSIGNAL | MSG_DONTWAIT;
+  if (more) flags |= MSG_MORE;
+  return send(fd, b, s, flags);
 }
 
 static long rcv(int fd, void* b, size_t s) {
@@ -384,8 +416,27 @@ pn_event_batch_t *pni_raw_connection_process(task_t *t, uint32_t io_events, bool
     else
       pni_task_wake_done(&rc->task);  // Complete task wake without event.
   }
+  if (io_events) {
+    rc->armed = false;
+    rc->current_arm = 0;
+  }
+  if (pni_raw_finished(&rc->raw_connection)) {
+    unlock(&rc->task.mutex);
+    praw_initiate_cleanup(rc);
+    return NULL;
+  }
   int events = io_events;
   int fd = rc->psocket.epoll_io.fd;
+
+  if (rc->first_schedule) {
+    rc->first_schedule = false;
+    assert(!events); // No socket yet.
+    assert(!rc->connected);
+    if (praw_connection_first_connect_lh(rc)) {
+      unlock(&rc->task.mutex);
+      return NULL;
+    }
+  }
   if (!rc->connected) {
     if (events & (EPOLLHUP | EPOLLERR)) {
       praw_connection_maybe_connect_lh(rc);
@@ -393,10 +444,10 @@ pn_event_batch_t *pni_raw_connection_process(task_t *t, uint32_t io_events, bool
     if (rc->disconnected) {
       pni_raw_connect_failed(&rc->raw_connection);
       unlock(&rc->task.mutex);
-      rc->batch_empty = false;
       return &rc->batch;
     }
     if (events & (EPOLLHUP | EPOLLERR)) {
+      // Continuation of praw_connection_maybe_connect_lh() logic.
       // A wake can be the first event.  Otherwise, wait for connection to complete.
       bool event_pending = task_wake || pni_raw_wake_is_pending(&rc->raw_connection) || pn_collector_peek(rc->raw_connection.collector);
       t->working = event_pending;
@@ -405,64 +456,88 @@ pn_event_batch_t *pni_raw_connection_process(task_t *t, uint32_t io_events, bool
     }
     if (events & EPOLLOUT)
       praw_connection_connected_lh(rc);
+
+    unlock(&rc->task.mutex);
+    return &rc->batch;
   }
   unlock(&rc->task.mutex);
 
-  if (events & EPOLLIN) pni_raw_read(&rc->raw_connection, fd, rcv, set_error);
+  if (events & EPOLLERR) {
+    // Read and write sides closed via RST.  Tear down immediately.
+    int soerr;
+    socklen_t soerrlen = sizeof(soerr);
+    int ec = getsockopt(fd, SOL_SOCKET, SO_ERROR, &soerr, &soerrlen);
+    if (ec == 0 && soerr) {
+      psocket_error(rc, soerr, "async disconnect");
+    }
+    pni_raw_async_disconnect(&rc->raw_connection);
+    return &rc->batch;
+  }
+  if (events & EPOLLHUP) {
+    rc->hup_detected = true;
+  }
+
+  if (events & (EPOLLIN | EPOLLRDHUP) || rc->read_check) {
+    pni_raw_read(&rc->raw_connection, fd, rcv, set_error);
+    rc->read_check = false;
+  }
   if (events & EPOLLOUT) pni_raw_write(&rc->raw_connection, fd, snd, set_error);
-  rc->batch_empty = false;
   return &rc->batch;
 }
 
 void pni_raw_connection_done(praw_connection_t *rc) {
   bool notify = false;
   bool ready = false;
-  bool have_event = false;
+  pn_raw_connection_t *raw = &rc->raw_connection;
+  int fd = rc->psocket.epoll_io.fd;
 
-  // If !batch_empty, can't be sure state machine up to date, so reschedule task if necessary.
-  if (!rc->batch_empty) {
-    if (pn_collector_peek(rc->raw_connection.collector))
-      have_event = true;
-    else {
-      pn_event_t *e = pni_raw_batch_next(&rc->batch);
-      // State machine up to date.
-      if (e) {
-        have_event = true;
-        // Sole event.  Can put back without order issues.
-        // Edge case, performance not important.
-        pn_collector_put(rc->raw_connection.collector, pn_event_class(e), pn_event_context(e), pn_event_type(e));
-      }
-    }
-  }
+  // Try write
+  if (pni_raw_can_write(raw)) pni_raw_write(raw, fd, snd, set_error);
+  pni_raw_process_shutdown(raw, fd, shutr, shutw);
+
+  // Update state machine and check for possible pending event.
+  bool have_event = pni_raw_batch_has_events(raw);
 
   lock(&rc->task.mutex);
+  bool wake_pending = pni_task_wake_pending(&rc->task) && pni_raw_can_wake(raw);
   pn_proactor_t *p = rc->task.proactor;
   tslot_t *ts = rc->task.runner;
   rc->task.working = false;
   // The task may be in the ready state even if we've got no raw connection
   // wakes outstanding because we dealt with it already in pni_raw_batch_next()
-  notify = (pni_task_wake_pending(&rc->task) || have_event) && schedule(&rc->task);
-  ready = rc->task.ready;
+  notify = (wake_pending || have_event) && schedule(&rc->task);
+  ready = rc->task.ready;  // No need to poll.  Already scheduled.
   unlock(&rc->task.mutex);
 
-  pn_raw_connection_t *raw = &rc->raw_connection;
-  int fd = rc->psocket.epoll_io.fd;
-  pni_raw_process_shutdown(raw, fd, shutr, shutw);
-  int wanted =
-    (pni_raw_can_read(raw)  ? EPOLLIN : 0) |
-    (pni_raw_can_write(raw) ? EPOLLOUT : 0);
-  if (wanted) {
-    rc->psocket.epoll_io.wanted = wanted;
-    rearm_polling(&rc->psocket.epoll_io, p->epollfd);  // TODO: check for error
+  if (pni_raw_finished(raw) && !ready) {
+    // If raw connection has no more work to do and safe to free resources, do so.
+    praw_initiate_cleanup(rc);
+  } else if (ready) {
+    // Already scheduled to run.  Skip poll.  Remember if we want a read.
+    rc->read_check = pni_raw_can_read(raw);
+  } else if (!rc->connected) {
+    // Connect logic has already armed the socket.
   } else {
-    bool finished_disconnect = raw->state==conn_fini && !ready && !raw->disconnectpending;
-    if (finished_disconnect) {
-      // If we're closed and we've sent the disconnect then close
-      pni_raw_finalize(raw);
-      praw_connection_cleanup(rc);
+    // Must poll for IO.
+    int wanted =
+      (pni_raw_can_read(raw)  ? (EPOLLIN | EPOLLRDHUP) : 0) |
+      (pni_raw_can_write(raw) ? EPOLLOUT : 0);
+
+    // wanted == 0 implies we block until either application wake() or EPOLLHUP | EPOLLERR.
+    // If wanted == 0 and hup_detected, blocking not possible, so skip arming until
+    // application provides read buffers.
+    if (wanted || !rc->hup_detected) {
+      // Arm only if there is a change.
+      if (!rc->armed || (wanted != rc->current_arm)) {
+        rc->psocket.epoll_io.wanted = rc->current_arm = wanted;
+        rc->armed = true;
+        rearm_polling(&rc->psocket.epoll_io, p->epollfd);  // TODO: check for error
+      }
     }
   }
 
+  // praw_initiate_cleanup() may have been called above. Can no longer touch rc or raw.
+
   lock(&p->sched_mutex);
   tslot_t *resume_thread;
   notify |= unassign_thread(p, ts, UNUSED, &resume_thread);
@@ -472,6 +547,6 @@ void pni_raw_connection_done(praw_connection_t *rc) {
 }
 
 void pni_raw_connection_forced_shutdown(praw_connection_t *rc) {
-  pni_raw_finalize(&rc->raw_connection);
-  praw_connection_cleanup(rc);
+  rc->armed = false;  // Tear down. No epoll event callbacks.
+  praw_initiate_cleanup(rc);
 }
