@@ -377,6 +377,19 @@ static inline pconnection_t *batch_pconnection(pn_event_batch_t *batch) {
     containerof(batch, pconnection_t, batch) : NULL;
 }
 
+pn_proactor_t *pn_event_batch_proactor(pn_event_batch_t *batch) {
+  return batch_proactor(batch);
+}
+
+pn_listener_t *pn_event_batch_listener(pn_event_batch_t *batch) {
+  return batch_listener(batch);
+}
+
+pn_connection_t *pn_event_batch_connection(pn_event_batch_t *batch) {
+  pconnection_t *r = batch_pconnection(batch);
+  return r ? r->driver.connection : NULL;
+}
+
 static inline work_t *batch_work(pn_event_batch_t *batch) {
   pconnection_t *pc = batch_pconnection(batch);
   if (pc) return &pc->work;
@@ -745,7 +758,7 @@ static bool leader_process_listener(pn_listener_t *l) {
       uv_safe_close((uv_handle_t*)&ls->tcp, on_close_lsocket);
     }
     /* NOTE: Fall through in case we have 0 sockets - e.g. resolver error */
-
+    PN_FALLTHROUGH;
    case L_CLOSING:              /* Closing - can we send PN_LISTENER_CLOSE? */
     if (!l->lsockets) {
       l->state = L_CLOSED;
@@ -1215,7 +1228,7 @@ static void work_free(work_t *w) {
   }
 }
 
-pn_proactor_t *pn_proactor() {
+pn_proactor_t *pn_proactor(void) {
   uv_once(&global_init_once, global_init_fn);
   pn_proactor_t *p = (pn_proactor_t*)calloc(1, sizeof(pn_proactor_t));
   p->collector = pn_collector();
@@ -1381,3 +1394,4 @@ void pn_raw_connection_read_close(pn_raw_connection_t *conn) {}
 void pn_raw_connection_write_close(pn_raw_connection_t *conn) {}
 const struct pn_netaddr_t *pn_raw_connection_local_addr(pn_raw_connection_t *connection) { return NULL; }
 const struct pn_netaddr_t *pn_raw_connection_remote_addr(pn_raw_connection_t *connection) { return NULL; }
+pn_raw_connection_t *pn_event_batch_raw_connection(pn_event_batch_t* batch) { return NULL; }
