@@ -475,7 +475,22 @@ void pni_raw_connect_failed(pn_raw_connection_t *conn) {
 }
 
 void pni_raw_wake(pn_raw_connection_t *conn) {
-  conn->wakepending = true;
+  if (conn->disconnect_state != disc_fini)
+    conn->wakepending = true;
+}
+
+bool pni_raw_wake_is_pending(pn_raw_connection_t *conn) {
+  return conn->wakepending;
+}
+
+bool pni_raw_can_wake(pn_raw_connection_t *conn) {
+  // True if DISCONNECTED event has not yet been generated.
+  return (conn->disconnect_state != disc_fini);
+}
+
+bool pni_raw_finished(pn_raw_connection_t *conn) {
+  // True if state machine is in final state and application has consumed final DISCONNECTED event.
+  return (conn->disconnect_state == disc_fini && pn_collector_peek(conn->collector) == NULL);
 }
 
 void pni_raw_read(pn_raw_connection_t *conn, int sock, long (*recv)(int, void*, size_t), void(*set_error)(pn_raw_connection_t *, const char *, int)) {
@@ -551,7 +566,7 @@ finished_reading:
   return;
 }
 
-void pni_raw_write(pn_raw_connection_t *conn, int sock, long (*send)(int, const void*, size_t), void(*set_error)(pn_raw_connection_t *, const char *, int)) {
+void pni_raw_write(pn_raw_connection_t *conn, int sock, long (*send)(int, const void*, size_t, bool), void(*set_error)(pn_raw_connection_t *, const char *, int)) {
   assert(conn);
 
   if (pni_raw_wdrained(conn)) return;
@@ -563,7 +578,8 @@ void pni_raw_write(pn_raw_connection_t *conn, int sock, long (*send)(int, const
     assert(conn->wbuffers[p-1].type == buff_unwritten);
     char *bytes = conn->wbuffers[p-1].bytes+conn->wbuffers[p-1].offset+conn->unwritten_offset;
     size_t s = conn->wbuffers[p-1].size-conn->unwritten_offset;
-    int r = send(sock,  bytes, s);
+    bool more = conn->wbuffers[p-1].next != 0;
+    int r = send(sock,  bytes, s, more);
     if (r < 0) {
       // Interrupted system call try again
       switch (errno) {
@@ -659,12 +675,14 @@ bool pni_raw_can_write(pn_raw_connection_t *conn) {
   return !pni_raw_wdrained(conn) && conn->wbuffer_first_towrite;
 }
 
-pn_event_t *pni_raw_event_next(pn_raw_connection_t *conn) {
+bool pni_raw_batch_has_events(pn_raw_connection_t *conn) {
+  // If collector empty, advance state machine as necessary and generate next event.
+  // Return true if at least one event is available.
   assert(conn);
   do {
-    pn_event_t *event = pn_collector_next(conn->collector);
+    pn_event_t *event = pn_collector_peek(conn->collector);
     if (event) {
-      return pni_log_event(conn, event);
+      return true;
     } else if (conn->connectpending) {
       pni_raw_put_event(conn, PN_RAW_CONNECTION_CONNECTED);
       conn->connectpending = false;
@@ -691,21 +709,7 @@ pn_event_t *pni_raw_event_next(pn_raw_connection_t *conn) {
         }
         conn->disconnect_state = disc_drain_msg;
         break;
-      // TODO: We'll leave the read/written events in here for the moment for backward compatibility
-      // remove them soon (after dispatch uses DRAIN_BUFFER)
       case disc_drain_msg:
-        if (conn->rbuffer_first_read) {
-          pni_raw_put_event(conn, PN_RAW_CONNECTION_READ);
-        }
-        conn->disconnect_state = disc_read_msg;
-        break;
-      case disc_read_msg:
-        if (conn->wbuffer_first_written) {
-          pni_raw_put_event(conn, PN_RAW_CONNECTION_WRITTEN);
-        }
-        conn->disconnect_state = disc_written_msg;
-        break;
-      case disc_written_msg:
         pni_raw_put_event(conn, PN_RAW_CONNECTION_DISCONNECTED);
         conn->disconnectpending = false;
         conn->disconnect_state = disc_fini;
@@ -720,11 +724,20 @@ pn_event_t *pni_raw_event_next(pn_raw_connection_t *conn) {
       pni_raw_put_event(conn, PN_RAW_CONNECTION_NEED_READ_BUFFERS);
       conn->rrequestedbuffers = true;
     } else {
-      return NULL;
+      return false;
     }
   } while (true);
 }
 
+pn_event_t *pni_raw_event_next(pn_raw_connection_t *conn) {
+  if (pni_raw_batch_has_events(conn)) {
+    pn_event_t* event = pn_collector_next(conn->collector);
+    assert(event);
+    return pni_log_event(conn, event);
+  }
+  return NULL;
+}
+
 void pni_raw_read_close(pn_raw_connection_t *conn) {
   // If already fully closed nothing to do
   if (pni_raw_rwclosed(conn)) {
@@ -785,6 +798,22 @@ void pni_raw_close(pn_raw_connection_t *conn) {
   }
 }
 
+void pni_raw_async_disconnect(pn_raw_connection_t *conn) {
+  if (pni_raw_rwclosed(conn))
+    return;
+
+  if (!pni_raw_rclosed(conn)) {
+    conn->state = pni_raw_new_state(conn, conn_read_closed);
+    conn->rclosedpending = true;
+  }
+  if (!pni_raw_wclosed(conn)) {
+    pni_raw_release_buffers(conn);
+    conn->state = pni_raw_new_state(conn, conn_write_closed);
+    conn->wclosedpending = true;
+  }
+  pni_raw_disconnect(conn);
+}
+
 bool pn_raw_connection_is_read_closed(pn_raw_connection_t *conn) {
   assert(conn);
   return pni_raw_rclosed(conn);
