@@ -30,6 +30,7 @@
 #include "platform/platform.h"
 #include "platform/platform_fmt.h"
 #include "core/util.h"
+#include <proton/annotations.h>
 #include <proton/error.h>
 #include <proton/tls.h>
 
@@ -222,7 +223,8 @@ struct pn_tls_t {
 static void encrypt(pn_tls_t *);
 static void decrypt(pn_tls_t *);
 static int pn_tls_alpn_cb(SSL *ssn, const unsigned char **out, unsigned char *outlen, const unsigned char *in, unsigned int inlen, void *arg);
-
+static void release_buffers(pn_tls_t *
+);
 
 static void tls_initialize_buffers(pn_tls_t *tls) {
   // Link together free lists
@@ -499,7 +501,8 @@ static X509 *get_peer_certificate(pn_tls_t *ssl);
 #define     PN_LEVEL_RAW       128
 #define     PN_LEVEL_ALL       65535
 
-static void ssl_log(void *v, int sev, const char *fmt, ...)
+PN_PRINTF_FORMAT_ATTR(3, 4)
+static void ssl_log(void *v, int sev, PN_PRINTF_FORMAT const char *fmt, ...)
 {
 #ifdef at_least_some_logging
   va_list ap;
@@ -520,7 +523,8 @@ static void ssl_log_flush(void *v, int sev)
   }
 }
 // log an error and dump the SSL error stack
-static void ssl_log_error(const char *fmt, ...)
+PN_PRINTF_FORMAT_ATTR(1, 2)
+static void ssl_log_error(PN_PRINTF_FORMAT const char *fmt, ...)
 {
 }
 
@@ -854,6 +858,10 @@ static bool pni_init_ssl_domain( pn_tls_config_t * domain, pn_tls_mode_t mode )
     return false;
   };
 
+  // Support intermediate/subordinate CAs as trust anchors.
+  X509_STORE* store = SSL_CTX_get_cert_store(domain->ctx);
+  X509_STORE_set_flags(store, X509_V_FLAG_PARTIAL_CHAIN);
+
   const long reject_insecure =
       SSL_OP_NO_SSLv2
     | SSL_OP_NO_SSLv3
@@ -1143,6 +1151,8 @@ int pn_tls_stop(pn_tls_t *tls) {
   if (tls->stopped)
     return PN_ARG_ERR;
   tls->stopped = true;
+  release_buffers(tls);
+  if (tls->validating) validate_strict(tls);
   return 0;
 }
 
@@ -1927,6 +1937,59 @@ static buff_ptr current_decrypted_result(pn_tls_t *tls) {
   return tls->dresult_first_blank;
 }
 
+static void release_buffers(pn_tls_t *tls) {
+  // encrypt input
+  for(;tls->encrypt_first_pending;) {
+    buff_ptr p = tls->encrypt_first_pending;
+    assert(tls->encrypt_buffers[p-1].type == buff_encrypt_pending);
+    if (!tls->encrypt_first_done) {
+      tls->encrypt_first_done = p;
+    }
+    if (tls->encrypt_last_done) {
+      tls->encrypt_buffers[tls->encrypt_last_done-1].next = p;
+    }
+    tls->encrypt_last_done = p;
+    tls->encrypt_first_pending = tls->encrypt_buffers[p-1].next;
+
+    tls->encrypt_buffers[p-1].next = 0;
+    tls->encrypt_buffers[p-1].type = buff_encrypt_done;
+  }
+
+  // decrypt input
+  for(;tls->decrypt_first_pending;) {
+    buff_ptr p = tls->decrypt_first_pending;
+    assert(tls->decrypt_buffers[p-1].type == buff_decrypt_pending);
+    if (!tls->decrypt_first_done) {
+      tls->decrypt_first_done = p;
+    }
+    if (tls->decrypt_last_done) {
+      tls->decrypt_buffers[tls->decrypt_last_done-1].next = p;
+    }
+    tls->decrypt_last_done = p;
+    tls->decrypt_first_pending = tls->decrypt_buffers[p-1].next;
+
+    tls->decrypt_buffers[p-1].next = 0;
+    tls->decrypt_buffers[p-1].type = buff_decrypt_done;
+  }
+
+  // encrypt output
+  for(;tls->eresult_first_blank;) {
+    buff_ptr p = tls->eresult_first_blank;
+    assert(tls->eresult_buffers[p-1].type == buff_eresult_blank);
+    blank_eresult_pop(tls, p);
+    encrypted_result_add(tls, p);
+  }
+
+  // decrypt output
+  for(;tls->dresult_first_blank;) {
+    buff_ptr p = tls->dresult_first_blank;
+    assert(tls->dresult_buffers[p-1].type == buff_dresult_blank);
+    blank_dresult_pop(tls, p);
+    decrypted_result_add(tls, p);
+  }
+
+}
+
 static bool try_shutdown(pn_tls_t *tls) {
   assert(tls->enc_closed && !tls->encrypt_first_pending && !tls->ssl_shutdown);
   bool success = false;
@@ -2056,7 +2119,8 @@ static void decrypt(pn_tls_t *tls) {
   pbuffer_t *pending = next_decrypt_pending(tls);
   bool peek_needed = false;
 
-  while (true) {
+  bool decrypt_done = false;
+  while (!decrypt_done) {
     if (tls->pn_tls_err)
       return;
 
@@ -2110,23 +2174,32 @@ static void decrypt(pn_tls_t *tls) {
     }
 
     // Done if not possible to move any more bytes from input to output bufs
-    if (tls->dec_closed) break;
-    if ((!pending || tls->dec_wblocked) // write side
-        && (!curr_result || tls->dec_rblocked)) // read side
-      break;
-  }
+    if ( (tls->dec_closed || !pending || tls->dec_wblocked) /* write side */ &&
+         (!curr_result || tls->dec_rblocked) ) /* read side */ {
+      decrypt_done = true;
+      if (peek_needed && !tls->pn_tls_err && !tls->dec_closed) {
+        // Set dec_rpending.
+        // Make OpenSSL process input to at least first decrypted byte (if any)
+        char unused;
+        int pcount = SSL_peek(tls->ssl, &unused, 1);
+        tls->dec_rpending = (pcount == 1);
+        if (pcount <= 0) {
+          check_error_reason(tls, pcount);
+        }
 
-  if (!tls->pn_tls_err && peek_needed) {
-    // Make OpenSSL examine the next buffered TLS record (if exists and complete)
-    char unused;
-    int pcount = SSL_peek(tls->ssl, &unused, 1);
-    tls->dec_rpending = (pcount == 1);
-    if (pcount <= 0) {
-      check_error_reason(tls, pcount);
+        // Peek may have made more room in buffer (i.e. handshake followed by large
+        // incomplete application record and dec_wblocked). If we did not process an
+        // application record, we must have processed at least one non-app record.
+        // No longer write blocked after peek.  PROTON-2736.
+        if (!tls->dec_rpending && tls->dec_wblocked) {
+          decrypt_done = false;
+          tls->dec_wblocked = false;
+        }
+      }
     }
   }
 
-  if (!tls->pn_tls_err && !tls->handshake_ok && SSL_do_handshake(tls->ssl) == 1) {
+  if (!tls->handshake_ok && SSL_do_handshake(tls->ssl) == 1) {
     tls->handshake_ok = true;
     tls->can_shutdown = true;
   }
